program asiasat(input,output);
{Lunar swingby of Asiasat 3 program written by
 Steven S. Pietrobon 4 May 1998, 17 Aug 2006.
 Copyright 1998-2006 Small World Communications}

const Re   =    6371000.; {m, mean radius of Earth}
      Rm   =    1738000.; {m, mean radius of Moon}
      Te   =      86164.; {m, sidereal rotation period of Earth}
      rem  =  384401000.; {m, mean Earth-Moon distance}
      mue  =    3.986e14; {m^3/s^2, gravity parameter of Earth}
      mum  =    4.903e12; {m^3/s^2, gravity parameter of Moon}
      hp1  =     365000.; {m, initial perigee}
      ha1  =   35989000.; {m, initial apogee}
      inc1 =        51.6; {degrees, initial inclination}

var rp,ra,a,e,vp,va,vcp,vca,pi:double;
    hg,vg,vp1,va1,dv1,vm,vp2,va2,dv2,vp3,va3,dv3,dv4,v3:double;
    vinf,hp3,ha3,theta,temp,alpha,inc,vc,hc:double;
    output_name:string;

procedure orbit(R,hp,ha,mu:double);
{finds orbital parameters}
begin{orbit}
  rp := R+hp;
  ra := R+ha;
  a := (ra+rp)/2.0;
  vp := sqrt(mu*ra/(a*rp));
  va := sqrt(mu*rp/(a*ra));
  vcp := sqrt(mu/rp);
  vca := sqrt(mu/ra);
end;{orbit}

function transfer(vi,vf,angle:double):double;
begin{transfer}
  transfer := sqrt(vi*vi + vf*vf - 2.0*vi*vf*cos(angle));
end;{transfer}

begin{asiasat}
  writeln;
  write('Enter output filename (return is standard output): ');
  readln(output_name);
  assign(output,output_name);
  rewrite(output);

  pi := 4.0*arctan(1.0);
  inc := pi*inc1/180;
  hg := exp(2.0*ln(sqrt(mue)*Te/(2.0*pi))/3.0) - Re;
  writeln;
  writeln('Geosynchrous altitiude = ',round(hg/1000),' km');
  orbit(Re,hp1,ha1,mue);
  vp1 := vp;
  va1 := va;
  orbit(Re,hg,ha1,mue);
  vg := vcp;
  vp2 := vp;
  va2 := va;
  dv1 := transfer(va1,va2,inc);
  dv2 := vp2 - vg;
  writeln('Geosynchronous speed = ',round(vg),' m/s');
  writeln('Initial perigee speed = ',round(vp1),' m/s');
  writeln('Initial apogee speed = ',round(va1),' m/s');
  writeln('Apogee delta V = ',round(dv1),' m/s');
  writeln('Apogee speed = ',round(va2),' m/s');
  writeln('Perigee speed = ',round(vp2),' m/s');
  writeln('Perigee delta V = ',round(dv2),' m/s');
  writeln('Toral delta V = ',round(dv1+dv2),' m/s');

  orbit(Re,hp1,rem-Re,mue);
  vm := vca;
  vp2 := vp;
  va2 := va;
  dv2 := vp2 - vp1;
  writeln;
  writeln('Lunar speed = ',round(vm),' m/s');
  writeln('Lunar transfer perigee speed = ',round(vp2),' m/s');
  writeln('Lunar transfer apogee speed = ',round(va2),' m/s');
  writeln('Lunar transfer delta V = ',round(dv2),' m/s');

  orbit(Re,hg,rem-Re,mue);
  vp3 := vp;
  va3 := va;
  dv3 := transfer(va2,va3,inc);
  dv4 := vp3 - vg;
  writeln;
  writeln('Geo transfer perigee speed = ',round(vp3),' m/s');
  writeln('Geo transfer apogee speed = ',round(va3),' m/s');
  writeln('Geo transfer apogee delta V (no swingby) = ',round(dv3),' m/s');
  writeln('Geo transfer perigee delta V (no swingby) = ',round(dv4),' m/s');
  writeln('Total delta V (no swingby) = ',round(dv2+dv3+dv4),' m/s');

  vinf := transfer(va2,vm,inc);
  writeln;
  writeln('Lunar relative speed = ',round(vinf),' m/s');
  temp := (vm - sqrt(mue*(Re+hg))/rem)/vinf;
  theta := arctan(sqrt(1/sqr(temp) - 1));
  temp := sqrt(2*(1.0 - (vm - va2*cos(inc))*cos(theta)/vinf));
  dv3 := vinf*temp;
  writeln('Lunar delta V (swingby) = ',round(dv3),' m/s');
  alpha := 2*arctan(1/sqrt(1/sqr(temp/2) - 1));
  vc := vinf/sqrt(1/sin(alpha/2) - 1);
  hc := mum/sqr(vc) - Rm;
  write('Theta = ',(180*theta/pi):5:2,' degrees, ');
  writeln('Alpha = ',(180*alpha/pi):5:2,' degrees');
  writeln('Lunar orbit circular speed = ',round(vc),' m/s');
  writeln('Lunar swingby minimum altitude = ',round(hc/1000),' km');

  v3 := transfer(vinf,vm,theta);
  writeln;
  writeln('Post swingby speed = ',round(v3),' m/s');
  a := 1.0/(2.0/rem - sqr(v3)/mue);
  e := 1.0 - sqr(rem*(vm - vinf*cos(theta)))/(a*mue);
  hp3 := a*(1.0 - e) - Re;
  ha3 := a*(1.0 + e) - Re;
  writeln('Post swingby orbit = ',round(hp3/1000),'x',round(ha3/1000),' km');
  orbit(Re,hp3,ha3,mue);
  vp3 := vp;
  dv4 := vp3 - vg;
  writeln('Post swingby perigee speed = ',round(vp3),' m/s');
  writeln('Post swingby perigee delta V = ',round(dv4),' m/s');
  writeln('Total delta V (swingby) = ',round(dv2+dv4),' m/s');
  close(output);
end.{asiasat}